"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const value_1 = require("./value");
const types_1 = require("./types");
function init(vm, valueTable) {
    initPrototype(vm, valueTable);
    initArrayConstructor(valueTable);
}
exports.init = init;
function initPrototype(vm, valueTable) {
    const metaObjectProto = new value_1.JSObject();
    value_1.JSObject.protoType = metaObjectProto;
    const metaFunctionProto = new value_1.JSNativeFunction(value_1.JSString.Empty, () => new value_1.JSFunction());
    value_1.JSFunction.protoType = metaFunctionProto;
    metaFunctionProto.setDescriptor(new value_1.JSString('__proto__'), new types_1.JSPropertyDescriptor(metaObjectProto, false));
    initMetaObjectProto(metaObjectProto);
    initMetaFunctionProto(vm, metaFunctionProto);
    const funcCtor = new value_1.JSNativeFunction(value_1.JSString.Empty, () => new value_1.JSObject());
    funcCtor.setDescriptor(new value_1.JSString('prototype'), new types_1.JSPropertyDescriptor(metaFunctionProto, false));
    const objectCtor = new value_1.JSNativeFunction(value_1.JSString.Empty, () => new value_1.JSObject());
    objectCtor.setDescriptor(new value_1.JSString('prototype'), new types_1.JSPropertyDescriptor(metaObjectProto, false));
    initObjectConstructor(objectCtor);
    valueTable.set('Function', funcCtor);
    valueTable.set('Object', objectCtor);
}
exports.initPrototype = initPrototype;
function initMetaObjectProto(metaObjectProto) { }
function initMetaFunctionProto(vm, metaFunctionProto) {
    metaFunctionProto.set(new value_1.JSString('call'), new value_1.JSBirdgeFunction(new value_1.JSString('call'), function (self, ...args) {
        if (this.isObject() && this.isFunction()) {
            vm.call(this, args, self);
            return;
        }
        throw new Error('is not callable');
    }));
    metaFunctionProto.set(new value_1.JSString('apply'), new value_1.JSBirdgeFunction(new value_1.JSString('apply'), function (self, args) {
        if (this.isObject() &&
            this.isFunction() &&
            args.isObject() &&
            args.isArray()) {
            vm.call(this, args.items, self);
            return;
        }
        throw new Error('is not callable');
    }));
    metaFunctionProto.set(new value_1.JSString('bind'), new value_1.JSNativeFunction(new value_1.JSString('bind'), function (self, ...args) {
        if (this.isObject() && this.isFunction()) {
            const callee = this;
            return new value_1.JSBirdgeFunction(value_1.JSString.Empty, function () {
                vm.call(callee, args, self);
            });
        }
        throw new Error('is not callable');
    }));
}
function initObjectConstructor(objectCtor) {
    objectCtor.set(new value_1.JSString('keys'), new value_1.JSNativeFunction(new value_1.JSString('keys'), obj => {
        if (obj.isObject()) {
            return new value_1.JSArray(Array.from(obj.properties.entries())
                .filter(([_, value]) => !!value.enumerable)
                .map(([key]) => new value_1.JSString(key.toString())));
        }
        return new value_1.JSArray([]);
    }));
}
function initArrayConstructor(valueTable) {
    const arrayCtor = new value_1.JSNativeFunction(new value_1.JSString('Array'), () => new value_1.JSArray([]));
    const araryProto = new value_1.JSObject();
    value_1.JSArray.protoType = araryProto;
    araryProto.setDescriptor(new value_1.JSString('length'), new types_1.JSPropertyDescriptor(undefined, false, true, new value_1.JSNativeFunction(new value_1.JSString(''), function () {
        if (this.isObject() && this.isArray()) {
            return new value_1.JSNumber(this.items.length);
        }
        return new value_1.JSNumber(0);
    })));
    valueTable.set('Array', arrayCtor);
}
//# sourceMappingURL=bom.js.map