import { OpCode, OpValue } from './opcode';
import { VObject, JSUndefined, JSFunction } from './value';
export interface VMDump {
    stack: VObject[];
    frames: StackFrame[];
    environments: Environment[];
    codes: (OpCode | OpValue)[];
    values: VObject[];
    cur: number;
}
export interface DoneResult {
    finished: true;
    value: VObject;
}
export interface StepResult {
    finished: false;
}
export declare type ExecResult = DoneResult | StepResult;
export declare enum EnvironmentType {
    lexer = 0,
    block = 1,
    global = 2
}
export interface GlobalEnvironment {
    type: EnvironmentType.global;
    valueTable: Map<string, VObject>;
}
export interface LexerEnvironment {
    type: EnvironmentType.lexer;
    valueTable: Map<string, VObject>;
    upValue: Map<string, VObject>;
}
export declare enum BlockEnvironmentKind {
    normal = 0,
    labeled = 1,
    iterable = 2
}
export interface BaseBlockEnvironment {
    type: EnvironmentType.block;
    kind: BlockEnvironmentKind;
    valueTable: Map<string, VObject>;
}
export interface NormalBlockEnvironment extends BaseBlockEnvironment {
    kind: BlockEnvironmentKind.normal;
}
export interface LabeledBlockEnvironment extends BaseBlockEnvironment {
    kind: BlockEnvironmentKind.labeled;
    label: string;
    end: number;
}
export interface IterableBlockEnviroment extends BaseBlockEnvironment {
    kind: BlockEnvironmentKind.iterable;
    end: number;
}
export declare type BlockEnvironment = BaseBlockEnvironment | LabeledBlockEnvironment | IterableBlockEnviroment;
export declare type Environment = LexerEnvironment | BlockEnvironment | GlobalEnvironment;
export interface StackFrame {
    ret: number;
    entry: number;
    environments: LexerEnvironment | GlobalEnvironment;
    thisObject: VObject | JSUndefined;
}
export declare class JSPropertyDescriptor {
    value?: VObject | undefined;
    enumerable: boolean;
    writable: boolean;
    getter?: JSFunction | undefined;
    setter?: JSFunction | undefined;
    constructor(value?: VObject | undefined, enumerable?: boolean, writable?: boolean, getter?: JSFunction | undefined, setter?: JSFunction | undefined);
}
export declare enum ObjectMemberType {
    property = 0,
    getter = 1,
    setter = 2
}
export interface Callable {
    call(callee: JSFunction, args: VObject[], thisObject: VObject): void;
}
